@@ -41,7 +41,7 @@
 static GMainLoop *loop = NULL;
 static PkBitfield roles = 0;
 static gboolean is_console = FALSE;
-static gboolean nowait = FALSE;
+static gboolean run_mainloop = TRUE;
 static PkControl *control = NULL;
 static PkTaskText *task = NULL;
 static PkProgressBar *progressbar = NULL;
@@ -176,7 +176,7 @@ pk_console_transaction_cb (PkTransactionPast *item, gpointer user_data)
 	g_print (" %s: %s\n", _("System time"), timespec);
 	/* TRANSLATORS: this is if the transaction succeeded or not */
 	g_print (" %s: %s\n", _("Succeeded"), succeeded ? _("True") : _("False"));
-	/* TRANSLATORS: this is the transactions role, e.g. "update-system" */
+	/* TRANSLATORS: this is the transactions role, e.g. "update-packages" */
 	g_print (" %s: %s\n", _("Role"), role_text);
 
 	/* only print if not null */
@@ -307,26 +307,27 @@ pk_console_update_detail_cb (PkUpdateDetail *item, gpointer data)
 {
 	gchar *package = NULL;
 	gchar *package_id;
-	gchar *updates;
-	gchar *obsoletes;
-	gchar *vendor_url;
-	gchar *bugzilla_url;
-	gchar *cve_url;
+	gchar **updates;
+	gchar **obsoletes;
+	gchar **vendor_urls;
+	gchar **bugzilla_urls;
+	gchar **cve_urls;
 	PkRestartEnum restart;
 	gchar *update_text;
 	gchar *changelog;
 	PkUpdateStateEnum state;
 	gchar *issued;
 	gchar *updated;
+	gchar *tmp;
 
 	/* get data */
 	g_object_get (item,
 		      "package-id", &package_id,
 		      "updates", &updates,
 		      "obsoletes", &obsoletes,
-		      "vendor-url", &vendor_url,
-		      "bugzilla-url", &bugzilla_url,
-		      "cve-url", &cve_url,
+		      "vendor-urls", &vendor_urls,
+		      "bugzilla-urls", &bugzilla_urls,
+		      "cve-urls", &cve_urls,
 		      "restart", &restart,
 		      "update-text", &update_text,
 		      "changelog", &changelog,
@@ -344,24 +345,34 @@ pk_console_update_detail_cb (PkUpdateDetail *item, gpointer data)
 	/* TRANSLATORS: details about the update, package name and version */
 	g_print (" %s: %s\n", _("Package"), package);
 	if (updates != NULL) {
+		tmp = g_strjoinv (", ", updates);
 		/* TRANSLATORS: details about the update, any packages that this update updates */
-		g_print (" %s: %s\n", _("Updates"), updates);
+		g_print (" %s: %s\n", _("Updates"), tmp);
+		g_free (tmp);
 	}
 	if (obsoletes != NULL) {
+		tmp = g_strjoinv (", ", obsoletes);
 		/* TRANSLATORS: details about the update, any packages that this update obsoletes */
-		g_print (" %s: %s\n", _("Obsoletes"), obsoletes);
+		g_print (" %s: %s\n", _("Obsoletes"), tmp);
+		g_free (tmp);
 	}
-	if (vendor_url != NULL) {
+	if (vendor_urls != NULL) {
+		tmp = g_strjoinv (", ", vendor_urls);
 		/* TRANSLATORS: details about the update, the vendor URLs */
-		g_print (" %s: %s\n", _("Vendor"), vendor_url);
+		g_print (" %s: %s\n", _("Vendor"), tmp);
+		g_free (tmp);
 	}
-	if (bugzilla_url != NULL) {
+	if (bugzilla_urls != NULL) {
+		tmp = g_strjoinv (", ", bugzilla_urls);
 		/* TRANSLATORS: details about the update, the bugzilla URLs */
-		g_print (" %s: %s\n", _("Bugzilla"), bugzilla_url);
+		g_print (" %s: %s\n", _("Bugzilla"), tmp);
+		g_free (tmp);
 	}
-	if (cve_url != NULL) {
+	if (cve_urls != NULL) {
+		tmp = g_strjoinv (", ", cve_urls);
 		/* TRANSLATORS: details about the update, the CVE URLs */
-		g_print (" %s: %s\n", _("CVE"), cve_url);
+		g_print (" %s: %s\n", _("CVE"), tmp);
+		g_free (tmp);
 	}
 	if (restart != PK_RESTART_ENUM_NONE) {
 		/* TRANSLATORS: details about the update, if the package requires a restart */
@@ -389,11 +400,11 @@ pk_console_update_detail_cb (PkUpdateDetail *item, gpointer data)
 	}
 	g_free (package);
 	g_free (package_id);
-	g_free (updates);
-	g_free (obsoletes);
-	g_free (vendor_url);
-	g_free (bugzilla_url);
-	g_free (cve_url);
+	g_strfreev (updates);
+	g_strfreev (obsoletes);
+	g_strfreev (vendor_urls);
+	g_strfreev (bugzilla_urls);
+	g_strfreev (cve_urls);
 	g_free (update_text);
 	g_free (changelog);
 	g_free (issued);
@@ -592,7 +603,7 @@ pk_console_progress_cb (PkProgress *progress, PkProgressType type, gpointer data
 		/* show new status on the bar */
 		text = pk_role_enum_to_localised_present (role);
 		if (!is_console) {
-			/* TRANSLATORS: the role is the point of the transaction, e.g. update-system */
+			/* TRANSLATORS: the role is the point of the transaction, e.g. update-packages */
 			g_print ("%s:\t%s\n", _("Transaction"), text);
 			goto out;
 		}
@@ -662,12 +673,15 @@ out:
 static void
 pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 {
-	PkError *error_code = NULL;
-	PkResults *results;
+	const gchar *filename;
+	gboolean ret;
 	GError *error = NULL;
+	GFile *file = NULL;
 	GPtrArray *array;
+	PkError *error_code = NULL;
 	PkPackageSack *sack;
 	PkRestartEnum restart;
+	PkResults *results;
 	PkRoleEnum role;
 
 	/* no more progress */
@@ -717,18 +731,18 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	array = pk_package_sack_get_array (sack);
 
 	/* package */
+	filename = g_object_get_data (G_OBJECT (task), "PkConsole:list-create-filename");
 	if (!is_console ||
 	    (role != PK_ROLE_ENUM_INSTALL_PACKAGES &&
 	     role != PK_ROLE_ENUM_UPDATE_PACKAGES &&
-	     role != PK_ROLE_ENUM_UPDATE_SYSTEM &&
-	     role != PK_ROLE_ENUM_REMOVE_PACKAGES)) {
+	     role != PK_ROLE_ENUM_REMOVE_PACKAGES &&
+	     filename == NULL)) {
 		g_ptr_array_foreach (array, (GFunc) pk_console_package_cb, NULL);
 	}
 
 	/* special case */
 	if (array->len == 0 &&
 	    (role == PK_ROLE_ENUM_GET_UPDATES ||
-	     role == PK_ROLE_ENUM_UPDATE_SYSTEM ||
 	     role == PK_ROLE_ENUM_UPDATE_PACKAGES)) {
 		/* TRANSLATORS: print a message when there are no updates */
 		g_print ("%s\n", _("There are no updates available at this time."));
@@ -832,9 +846,23 @@ pk_console_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 		/* TRANSLATORS: a package needs to restart the session (due to security) */
 		g_print ("%s\n", _("Please logout and login to complete the update as important security updates have been installed."));
 	}
+
+	/* write the sack to disk */
+	if (role == PK_ROLE_ENUM_GET_PACKAGES && filename != NULL) {
+		file = g_file_new_for_path (filename);
+		ret = pk_package_sack_to_file (sack, file, &error);
+		if (!ret) {
+			g_print ("%s: %s\n", _("Fatal error"), error->message);
+			g_error_free (error);
+			retval = PK_EXIT_CODE_TRANSACTION_FAILED;
+			goto out;
+		}
+	}
 out:
 	if (error_code != NULL)
 		g_object_unref (error_code);
+	if (file != NULL)
+		g_object_unref (file);
 	if (results != NULL)
 		g_object_unref (results);
 	g_main_loop_quit (loop);
@@ -950,8 +978,15 @@ pk_console_update_packages (gchar **packages, GError **error)
 	gboolean ret = TRUE;
 	gchar **package_ids;
 	GError *error_local = NULL;
+	PkBitfield filters;
 
-	package_ids = pk_console_resolve_packages (PK_CLIENT(task), pk_bitfield_value (PK_FILTER_ENUM_NOT_INSTALLED), packages, &error_local);
+	filters = pk_bitfield_from_enums (PK_FILTER_ENUM_NOT_INSTALLED,
+					  PK_FILTER_ENUM_NEWEST,
+					  -1);
+	package_ids = pk_console_resolve_packages (PK_CLIENT(task),
+						   filters,
+						   packages,
+						   &error_local);
 	if (package_ids == NULL) {
 		/* TRANSLATORS: There was an error getting the list of files for the package. The detailed error follows */
 		*error = g_error_new (1, 0, _("This tool could not find the package: %s"), error_local->message);
@@ -969,6 +1004,52 @@ out:
 	return ret;
 }
 
+/**
+ * pk_console_update_system:
+ **/
+static gboolean
+pk_console_update_system (PkBitfield filters, GError **error)
+{
+	gboolean ret = TRUE;
+	gchar **package_ids = NULL;
+	PkPackageSack *sack = NULL;
+	PkResults *results;
+
+	/* get the current updates */
+	pk_bitfield_add (filters, PK_FILTER_ENUM_NEWEST);
+	results = pk_task_get_updates_sync (PK_TASK (task),
+					    filters,
+					    cancellable,
+					    (PkProgressCallback) pk_console_progress_cb, NULL,
+					    error);
+	if (results == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+
+	/* do the async action */
+	sack = pk_results_get_package_sack (results);
+	package_ids = pk_package_sack_get_ids (sack);
+	if (g_strv_length (package_ids) == 0) {
+		pk_progress_bar_end (progressbar);
+		/* TRANSLATORS: there are no updates, so nothing to do */
+		g_print ("%s\n", _("No packages require updating to newer versions."));
+		retval = PK_EXIT_CODE_NOTHING_USEFUL;
+		ret = FALSE;
+		goto out;
+	}
+	pk_task_update_packages_async (PK_TASK(task), package_ids, cancellable,
+				       (PkProgressCallback) pk_console_progress_cb, NULL,
+				       (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+out:
+	if (sack != NULL)
+		g_object_unref (sack);
+	if (results != NULL)
+		g_object_unref (results);
+	g_strfreev (package_ids);
+	return ret;
+}
+
 /**
  * pk_console_get_requires:
  **/
@@ -979,7 +1060,7 @@ pk_console_get_requires (PkBitfield filters, gchar **packages, GError **error)
 	gchar **package_ids = NULL;
 	GError *error_local = NULL;
 
-	package_ids = pk_console_resolve_packages (PK_CLIENT(task), pk_bitfield_value (PK_FILTER_ENUM_NONE), packages, &error_local);
+	package_ids = pk_console_resolve_packages (PK_CLIENT(task), filters, packages, &error_local);
 	if (package_ids == NULL) {
 		/* TRANSLATORS: There was an error getting the list of files for the package. The detailed error follows */
 		*error = g_error_new (1, 0, _("This tool could not find all the packages: %s"), error_local->message);
@@ -989,7 +1070,11 @@ pk_console_get_requires (PkBitfield filters, gchar **packages, GError **error)
 	}
 
 	/* do the async action */
-	pk_task_get_requires_async (PK_TASK (task),filters, package_ids, TRUE, cancellable,
+	pk_task_get_requires_async (PK_TASK (task),
+				    pk_bitfield_value (PK_FILTER_ENUM_INSTALLED),
+				    package_ids,
+				    TRUE,
+				    cancellable,
 				    (PkProgressCallback) pk_console_progress_cb, NULL,
 				    (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 out:
@@ -1183,11 +1268,10 @@ pk_console_get_summary (void)
 		g_string_append_printf (string, "  %s\n", "install-sig [type] [key_id] [package_id]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REMOVE_PACKAGES))
 		g_string_append_printf (string, "  %s\n", "remove [package]");
-	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_SYSTEM) ||
-	    pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_PACKAGES))
+	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_UPDATE_PACKAGES))
 		g_string_append_printf (string, "  %s\n", "update <package>");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REFRESH_CACHE))
-		g_string_append_printf (string, "  %s\n", "refresh [--force]");
+		g_string_append_printf (string, "  %s\n", "refresh [force]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_RESOLVE))
 		g_string_append_printf (string, "  %s\n", "resolve [package]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_UPDATES))
@@ -1206,8 +1290,6 @@ pk_console_get_summary (void)
 		g_string_append_printf (string, "  %s\n", "get-update-detail [package]");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_PACKAGES))
 		g_string_append_printf (string, "  %s\n", "get-packages");
-	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_ROLLBACK))
-		g_string_append_printf (string, "  %s\n", "rollback");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_GET_REPO_LIST))
 		g_string_append_printf (string, "  %s\n", "repo-list");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REPO_ENABLE)) {
@@ -1224,6 +1306,11 @@ pk_console_get_summary (void)
 		g_string_append_printf (string, "  %s\n", "get-categories");
 	if (pk_bitfield_contain (roles, PK_ROLE_ENUM_REPAIR_SYSTEM))
 		g_string_append_printf (string, "  %s\n", "repair");
+#ifdef PK_HAS_OFFLINE_UPDATES
+	g_string_append_printf (string, "  %s\n", "offline-get-prepared");
+	g_string_append_printf (string, "  %s\n", "offline-trigger");
+	g_string_append_printf (string, "  %s\n", "offline-status");
+#endif
 	return g_string_free (string, FALSE);
 }
 
@@ -1250,6 +1337,129 @@ out:
 	g_main_loop_quit (loop);
 }
 
+/**
+ * pk_console_offline_get_prepared:
+ **/
+static gboolean
+pk_console_offline_get_prepared (GError **error)
+{
+	gboolean ret;
+	gchar *data = NULL;
+	gchar **split = NULL;
+	gchar *tmp;
+	guint i;
+
+	/* get data */
+	ret = g_file_get_contents ("/var/lib/PackageKit/prepared-update",
+				   &data, NULL, NULL);
+	if (!ret) {
+		g_set_error_literal (error,
+				     1,
+				     PK_EXIT_CODE_FILE_NOT_FOUND,
+				     "No offline updates have been prepared");
+		goto out;
+	}
+	split = g_strsplit (data, "\n", -1);
+	g_print ("Prepared updates:\n");
+	for (i = 0; split[i] != NULL; i++) {
+		tmp = pk_package_id_to_printable (split[i]);
+		g_print ("Update %02i: %s\n", i, tmp);
+		g_free (tmp);
+	}
+out:
+	g_free (data);
+	return ret;
+}
+
+/**
+ * pk_console_offline_trigger:
+ **/
+static gboolean
+pk_console_offline_trigger (GError **error)
+{
+	gboolean ret;
+	gchar *cmdline;
+
+	cmdline = g_strdup_printf ("pkexec %s/pk-trigger-offline-update", LIBEXECDIR);
+	ret = g_spawn_command_line_sync (cmdline,
+					 NULL,
+					 NULL,
+					 NULL,
+					 error);
+	g_free (cmdline);
+	return ret;
+}
+
+#define PK_OFFLINE_UPDATE_RESULTS	"PackageKit Offline Update Results"
+
+/**
+ * pk_console_offline_status:
+ **/
+static gboolean
+pk_console_offline_status (GError **error)
+{
+	gboolean ret;
+	gboolean success;
+	gchar *data = NULL;
+	gchar **split = NULL;
+	gchar *tmp;
+	GKeyFile *file;
+	guint i;
+
+	/* load data */
+	file = g_key_file_new ();
+	ret = g_key_file_load_from_file (file,
+					 "/var/lib/PackageKit/offline-update-competed",
+					 G_KEY_FILE_NONE,
+					 NULL);
+	if (!ret) {
+		g_set_error_literal (error,
+				     1,
+				     PK_EXIT_CODE_FILE_NOT_FOUND,
+				     "No offline updates have been processed");
+		goto out;
+	}
+
+	/* did it succeed */
+	success = g_key_file_get_boolean (file,
+					  PK_OFFLINE_UPDATE_RESULTS,
+					  "Success",
+					  NULL);
+	if (!success) {
+		g_print ("Status:\tFailed\n");
+		tmp = g_key_file_get_string (file,
+					     PK_OFFLINE_UPDATE_RESULTS,
+					     "ErrorCode",
+					     NULL);
+		if (tmp != NULL)
+			g_print ("ErrorCode:\%s\n", tmp);
+		g_free (tmp);
+		tmp = g_key_file_get_string (file,
+					     PK_OFFLINE_UPDATE_RESULTS,
+					     "ErrorDetails",
+					     NULL);
+		if (tmp != NULL)
+			g_print ("ErrorDetails:\%s\n", tmp);
+		g_free (tmp);
+	} else {
+		g_print ("Status:\tSuccess\n");
+		data = g_key_file_get_string (file,
+					      PK_OFFLINE_UPDATE_RESULTS,
+					      "Packages",
+					      NULL);
+		split = g_strsplit (data, ";", -1);
+		for (i = 0; split[i] != NULL; i++) {
+			tmp = pk_package_id_to_printable (split[i]);
+			g_print ("Updated %02i: %s\n", i, tmp);
+			g_free (tmp);
+		}
+	}
+out:
+	g_key_file_free (file);
+	g_free (data);
+	return ret;
+}
+
 /**
  * main:
  **/
@@ -1261,13 +1471,13 @@ main (int argc, char *argv[])
 	GError *error_local = NULL;
 	gboolean background = FALSE;
 	gboolean noninteractive = FALSE;
+	gboolean only_download = FALSE;
 	guint cache_age = 0;
 	gboolean plain = FALSE;
 	gboolean program_version = FALSE;
 	GOptionContext *context;
 	gchar *options_help;
 	gchar *filter = NULL;
-	gchar *root = NULL;
 	gchar *summary = NULL;
 	const gchar *mode;
 	const gchar *http_proxy;
@@ -1286,15 +1496,12 @@ main (int argc, char *argv[])
 		{ "filter", '\0', 0, G_OPTION_ARG_STRING, &filter,
 			/* TRANSLATORS: command line argument, use a filter to narrow down results */
 			_("Set the filter, e.g. installed"), NULL},
-		{ "root", '\0', 0, G_OPTION_ARG_STRING, &root,
-			/* TRANSLATORS: command line argument, use a non-standard install prefix */
-			_("Set the install root, e.g. '/' or '/mnt/ltsp'"), NULL},
-		{ "nowait", 'n', 0, G_OPTION_ARG_NONE, &nowait,
-			/* TRANSLATORS: command line argument, work asynchronously */
-			_("Exit without waiting for actions to complete"), NULL},
 		{ "noninteractive", 'y', 0, G_OPTION_ARG_NONE, &noninteractive,
 			/* command line argument, do we ask questions */
 			_("Install the packages without asking for confirmation"), NULL },
+		{ "only-download", 'y', 0, G_OPTION_ARG_NONE, &only_download,
+			/* command line argument, do we just download or apply changes */
+			_("Prepare the transaction by downloading pakages only"), NULL },
 		{ "background", 'n', 0, G_OPTION_ARG_NONE, &background,
 			/* TRANSLATORS: command line argument, this command is not a priority */
 			_("Run the command using idle network bandwidth and also using less power"), NULL},
@@ -1316,7 +1523,9 @@ main (int argc, char *argv[])
 	if (! g_thread_supported ())
 		g_thread_init (NULL);
 #endif
+#if (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 35)
 	g_type_init ();
+#endif
 
 	/* do stuff on ctrl-c */
 	signal (SIGINT, pk_console_sigint_cb);
@@ -1385,8 +1594,9 @@ main (int argc, char *argv[])
 	task = pk_task_text_new ();
 	g_object_set (task,
 		      "background", background,
-		      "simulate", !noninteractive,
+		      "simulate", !noninteractive && !only_download,
 		      "interactive", !noninteractive,
+		      "only-download", only_download,
 		      "cache-age", cache_age,
 		      NULL);
 
@@ -1405,18 +1615,6 @@ main (int argc, char *argv[])
 		}
 	}
 
-	/* set the install root if set */
-	if (root != NULL) {
-		ret = pk_control_set_root (control, root, NULL, &error_local);
-		if (!ret) {
-			/* TRANSLATORS: The user specified an incorrect filter */
-			error = g_error_new (1, 0, "%s: %s", _("The install root could not be set"), error_local->message);
-			g_error_free (error_local);
-			retval = PK_EXIT_CODE_CANNOT_SETUP;
-			goto out;
-		}
-	}
-
 	/* check filter */
 	if (filter != NULL) {
 		filters = pk_filter_bitfield_from_string (filter);
@@ -1437,6 +1635,9 @@ main (int argc, char *argv[])
 	if (argc > 4)
 		parameter = argv[4];
 
+	/* start polkit tty agent to listen for password requests */
+	pk_polkit_agent_open ();
+
 	/* parse the big list */
 	if (strcmp (mode, "search") == 0) {
 		if (value == NULL) {
@@ -1504,7 +1705,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_install_packages (argv+2, &error);
+		run_mainloop = pk_console_install_packages (argv+2, &error);
 
 	} else if (strcmp (mode, "install-local") == 0) {
 		if (value == NULL) {
@@ -1535,7 +1736,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_remove_packages (argv+2, &error);
+		run_mainloop = pk_console_remove_packages (argv+2, &error);
 
 	} else if (strcmp (mode, "download") == 0) {
 		if (value == NULL || details == NULL) {
@@ -1551,7 +1752,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_FILE_NOT_FOUND;
 			goto out;
 		}
-		nowait = !pk_console_download_packages (argv+3, value, &error);
+		run_mainloop = pk_console_download_packages (argv+3, value, &error);
 
 	} else if (strcmp (mode, "accept-eula") == 0) {
 		if (value == NULL) {
@@ -1564,25 +1765,12 @@ main (int argc, char *argv[])
 					     (PkProgressCallback) pk_console_progress_cb, NULL,
 					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
-	} else if (strcmp (mode, "rollback") == 0) {
-		if (value == NULL) {
-			/* TRANSLATORS: geeky error, 99.9999% of users won't see this */
-			error = g_error_new (1, 0, "%s", _("A transaction identifier (tid) is required"));
-			retval = PK_EXIT_CODE_SYNTAX_INVALID;
-			goto out;
-		}
-		pk_task_rollback_async (PK_TASK (task),value, cancellable,
-					(PkProgressCallback) pk_console_progress_cb, NULL,
-					(GAsyncReadyCallback) pk_console_finished_cb, NULL);
-
 	} else if (strcmp (mode, "update") == 0) {
 		if (value == NULL) {
 			/* do the system update */
-			pk_task_update_system_async (PK_TASK(task), cancellable,
-						     (PkProgressCallback) pk_console_progress_cb, NULL,
-						     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
+			run_mainloop = pk_console_update_system (filters, &error);
 		} else {
-			nowait = !pk_console_update_packages (argv+2, &error);
+			run_mainloop = pk_console_update_packages (argv+2, &error);
 		}
 
 	} else if (strcmp (mode, "resolve") == 0) {
@@ -1638,7 +1826,7 @@ main (int argc, char *argv[])
 		PkRoleEnum role;
 		if (value == NULL) {
 			/* TRANSLATORS: The user didn't specify what action to use */
-			error = g_error_new (1, 0, "%s", _("An action, e.g. 'update-system' is required"));
+			error = g_error_new (1, 0, "%s", _("An action, e.g. 'update-packages' is required"));
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
@@ -1659,7 +1847,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_get_depends (filters, argv+2, &error);
+		run_mainloop = pk_console_get_depends (filters, argv+2, &error);
 
 	} else if (strcmp (mode, "get-distro-upgrades") == 0) {
 		pk_client_get_distro_upgrades_async (PK_CLIENT(task), cancellable,
@@ -1673,7 +1861,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_get_update_detail (argv+2, &error);
+		run_mainloop = pk_console_get_update_detail (argv+2, &error);
 
 	} else if (strcmp (mode, "get-requires") == 0) {
 		if (value == NULL) {
@@ -1682,7 +1870,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_get_requires (filters, argv+2, &error);
+		run_mainloop = pk_console_get_requires (filters, argv+2, &error);
 
 	} else if (strcmp (mode, "what-provides") == 0) {
 		if (value == NULL) {
@@ -1702,7 +1890,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_get_details (argv+2, &error);
+		run_mainloop = pk_console_get_details (argv+2, &error);
 
 	} else if (strcmp (mode, "get-files") == 0) {
 		if (value == NULL) {
@@ -1711,7 +1899,7 @@ main (int argc, char *argv[])
 			retval = PK_EXIT_CODE_SYNTAX_INVALID;
 			goto out;
 		}
-		nowait = !pk_console_get_files (argv+2, &error);
+		run_mainloop = pk_console_get_files (argv+2, &error);
 
 	} else if (strcmp (mode, "get-updates") == 0) {
 		pk_task_get_updates_async (PK_TASK (task),filters, cancellable,
@@ -1752,7 +1940,7 @@ main (int argc, char *argv[])
 		g_strdelimit (text, ";", '\n');
 		g_print ("%s\n", text);
 		g_free (text);
-		nowait = TRUE;
+		run_mainloop = FALSE;
 
 	} else if (strcmp (mode, "get-filters") == 0) {
 		g_object_get (control,
@@ -1762,7 +1950,27 @@ main (int argc, char *argv[])
 		g_strdelimit (text, ";", '\n');
 		g_print ("%s\n", text);
 		g_free (text);
-		nowait = TRUE;
+		run_mainloop = FALSE;
+
+	} else if (strcmp (mode, "backend-details") == 0) {
+		gchar *backend_author = NULL;
+		gchar *backend_description = NULL;
+		gchar *backend_name = NULL;
+		g_object_get (control,
+			      "backend-author", &backend_author,
+			      "backend-description", &backend_description,
+			      "backend-name", &backend_name,
+			      NULL);
+		if (backend_name != NULL && backend_name[0] != '\0')
+			g_print ("Name:\t\t%s\n", backend_name);
+		if (backend_description != NULL && backend_description[0] != '\0')
+			g_print ("Description:\t%s\n", backend_description);
+		if (backend_author != NULL && backend_author[0] != '\0')
+			g_print ("Author:\t\t%s\n", backend_author);
+		g_free (backend_name);
+		g_free (backend_description);
+		g_free (backend_author);
+		run_mainloop = FALSE;
 
 	} else if (strcmp (mode, "get-groups") == 0) {
 		g_object_get (control,
@@ -1772,7 +1980,28 @@ main (int argc, char *argv[])
 		g_strdelimit (text, ";", '\n');
 		g_print ("%s\n", text);
 		g_free (text);
-		nowait = TRUE;
+		run_mainloop = FALSE;
+
+	} else if (strcmp (mode, "offline-get-prepared") == 0) {
+
+		run_mainloop = FALSE;
+		ret = pk_console_offline_get_prepared (&error);
+		if (!ret)
+			retval = error->code;
+
+	} else if (strcmp (mode, "offline-trigger") == 0) {
+
+		run_mainloop = FALSE;
+		ret = pk_console_offline_trigger (&error);
+		if (!ret)
+			retval = error->code;
+
+	} else if (strcmp (mode, "offline-status") == 0) {
+
+		run_mainloop = FALSE;
+		ret = pk_console_offline_status (&error);
+		if (!ret)
+			retval = error->code;
 
 	} else if (strcmp (mode, "get-transactions") == 0) {
 		pk_client_get_old_transactions_async (PK_CLIENT(task), 10, cancellable,
@@ -1780,7 +2009,7 @@ main (int argc, char *argv[])
 						      (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
 	} else if (strcmp (mode, "refresh") == 0) {
-		gboolean force = value && !strcmp (value, "--force");
+		gboolean force = (value != NULL && g_strcmp0 (value, "force") == 0);
 		pk_task_refresh_cache_async (PK_TASK (task), force, cancellable,
 					     (PkProgressCallback) pk_console_progress_cb, NULL,
 					     (GAsyncReadyCallback) pk_console_finished_cb, NULL);
@@ -1790,13 +2019,39 @@ main (int argc, char *argv[])
 		                             (PkProgressCallback) pk_console_progress_cb, NULL,
 		                             (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 
+	} else if (strcmp (mode, "list-create") == 0) {
+		if (value == NULL) {
+			/* TRANSLATORS: The user did not provide a distro name */
+			error = g_error_new (1, 0, "%s", _("You need to specify a list file to create"));
+			retval = PK_EXIT_CODE_SYNTAX_INVALID;
+			goto out;
+		}
+
+		/* file exists */
+		ret = g_file_test (value, G_FILE_TEST_EXISTS);
+		if (ret) {
+			/* TRANSLATORS: There was an error getting the list of packages. The filename follows */
+			error = g_error_new (1, 0, _("File already exists: %s"), value);
+			retval = PK_EXIT_CODE_FILE_NOT_FOUND;
+			goto out;
+			return FALSE;
+		}
+
+		/* get package list */
+		g_object_set_data_full (G_OBJECT (task),
+					"PkConsole:list-create-filename",
+					g_strdup (value),
+					g_free);
+		pk_task_get_packages_async (PK_TASK (task), filters, cancellable,
+					    (PkProgressCallback) pk_console_progress_cb, NULL,
+					    (GAsyncReadyCallback) pk_console_finished_cb, NULL);
 	} else {
 		/* TRANSLATORS: The user tried to use an unsupported option on the command line */
 		error = g_error_new (1, 0, _("Option '%s' is not supported"), mode);
 	}
 
 	/* do we wait for the method? */
-	if (!nowait && error == NULL)
+	if (run_mainloop && error == NULL)
 		g_main_loop_run (loop);
 
 out:
@@ -1807,9 +2062,11 @@ out:
 			retval = EXIT_FAILURE;
 	}
 
+	/* stop listening for polkit questions */
+	pk_polkit_agent_close ();
+
 	g_free (options_help);
 	g_free (filter);
-	g_free (root);
 	g_free (summary);
 	g_object_unref (progressbar);
 	g_object_unref (control);
