@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2012 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -38,6 +38,8 @@ static void     pk_task_finalize	(GObject     *object);
 
 #define PK_TASK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PK_TYPE_TASK, PkTaskPrivate))
 
+#define PK_TASK_TRANSACTION_CANCELLED_RETRY_TIMEOUT	2000 /* ms */
+
 /**
  * PkTaskPrivate:
  *
@@ -47,12 +49,14 @@ struct _PkTaskPrivate
 {
 	GPtrArray			*array;
 	gboolean			 simulate;
+	gboolean			 only_download;
 	gboolean			 interactive;
 };
 
 enum {
 	PROP_0,
 	PROP_SIMULATE,
+	PROP_ONLY_PREPARE,
 	PROP_INTERACTIVE,
 	PROP_LAST
 };
@@ -67,7 +71,8 @@ typedef struct {
 	PkRoleEnum			 role;
 	PkExitEnum			 exit_enum;
 	gboolean			 simulate;
-	gboolean			 only_trusted;
+	gboolean			 only_download;
+	gboolean			 transaction_flags;
 	gchar				**package_ids;
 	gboolean			 allow_deps;
 	gboolean			 autoremove;
@@ -89,6 +94,7 @@ typedef struct {
 	gchar				**values;
 	PkBitfield			 filters;
 	PkProvidesEnum			 provides;
+	guint				 retry_id;
 } PkTaskState;
 
 G_DEFINE_TYPE (PkTask, pk_task, PK_TYPE_CLIENT)
@@ -119,7 +125,7 @@ pk_task_find_by_request (PkTask *task, guint request)
 	g_return_val_if_fail (request != 0, NULL);
 
 	array = task->priv->array;
-	for (i=0; i<array->len; i++) {
+	for (i = 0; i < array->len; i++) {
 		item = g_ptr_array_index (array, i);
 		if (item->request == request)
 			goto out;
@@ -156,6 +162,8 @@ pk_task_generic_state_finish (PkTaskState *state, const GError *error)
 		g_object_unref (state->results);
 		state->results = NULL;
 	}
+	if (state->retry_id != 0)
+		g_source_remove (state->retry_id);
 	g_free (state->directory);
 	g_free (state->repo_id);
 	g_free (state->transaction_id);
@@ -174,42 +182,53 @@ pk_task_generic_state_finish (PkTaskState *state, const GError *error)
 static void
 pk_task_do_async_action (PkTaskState *state)
 {
+	PkBitfield transaction_flags;
+
 	/* so the callback knows if we are serious or not */
 	state->simulate = FALSE;
 
+	/* only prepare the transaction */
+	transaction_flags = state->transaction_flags;
+	if (state->task->priv->only_download) {
+		pk_bitfield_add (transaction_flags,
+				 PK_TRANSACTION_FLAG_ENUM_ONLY_DOWNLOAD);
+	}
+
 	/* do the correct action */
 	if (state->role == PK_ROLE_ENUM_INSTALL_PACKAGES) {
-		pk_client_install_packages_async (PK_CLIENT(state->task), state->only_trusted, state->package_ids,
+		pk_client_install_packages_async (PK_CLIENT(state->task), transaction_flags, state->package_ids,
 						  state->cancellable, state->progress_callback, state->progress_user_data,
 						  (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_UPDATE_PACKAGES) {
-		pk_client_update_packages_async (PK_CLIENT(state->task), state->only_trusted, state->package_ids,
+		pk_client_update_packages_async (PK_CLIENT(state->task), transaction_flags, state->package_ids,
 						 state->cancellable, state->progress_callback, state->progress_user_data,
 						 (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_REMOVE_PACKAGES) {
-		pk_client_remove_packages_async (PK_CLIENT(state->task), state->package_ids, state->allow_deps, state->autoremove,
-						 state->cancellable, state->progress_callback, state->progress_user_data,
+		pk_client_remove_packages_async (PK_CLIENT(state->task),
+						 transaction_flags,
+						 state->package_ids,
+						 state->allow_deps,
+						 state->autoremove,
+						 state->cancellable,
+						 state->progress_callback,
+						 state->progress_user_data,
 						 (GAsyncReadyCallback) pk_task_ready_cb, state);
-	} else if (state->role == PK_ROLE_ENUM_UPDATE_SYSTEM) {
-		pk_client_update_system_async (PK_CLIENT(state->task), state->only_trusted,
-					       state->cancellable, state->progress_callback, state->progress_user_data,
-					       (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_INSTALL_FILES) {
-		pk_client_install_files_async (PK_CLIENT(state->task), state->only_trusted, state->files,
+		pk_client_install_files_async (PK_CLIENT(state->task), transaction_flags, state->files,
 					       state->cancellable, state->progress_callback, state->progress_user_data,
 					       (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_RESOLVE) {
 		pk_client_resolve_async (PK_CLIENT(state->task), state->filters, state->packages,
-				         state->cancellable, state->progress_callback, state->progress_user_data,
-				         (GAsyncReadyCallback) pk_task_ready_cb, state);
+					 state->cancellable, state->progress_callback, state->progress_user_data,
+					 (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_SEARCH_NAME) {
 		pk_client_search_names_async (PK_CLIENT(state->task), state->filters, state->values,
 					      state->cancellable, state->progress_callback, state->progress_user_data,
 					      (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_SEARCH_DETAILS) {
 		pk_client_search_details_async (PK_CLIENT(state->task), state->filters, state->values,
-					        state->cancellable, state->progress_callback, state->progress_user_data,
-					        (GAsyncReadyCallback) pk_task_ready_cb, state);
+						state->cancellable, state->progress_callback, state->progress_user_data,
+						(GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_SEARCH_GROUP) {
 		pk_client_search_groups_async (PK_CLIENT(state->task), state->filters, state->values,
 					       state->cancellable, state->progress_callback, state->progress_user_data,
@@ -256,16 +275,12 @@ pk_task_do_async_action (PkTaskState *state)
 					   (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_GET_CATEGORIES) {
 		pk_client_get_categories_async (PK_CLIENT(state->task),
-					        state->cancellable, state->progress_callback, state->progress_user_data,
-					        (GAsyncReadyCallback) pk_task_ready_cb, state);
+						state->cancellable, state->progress_callback, state->progress_user_data,
+						(GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_REFRESH_CACHE) {
 		pk_client_refresh_cache_async (PK_CLIENT(state->task), state->force,
 					       state->cancellable, state->progress_callback, state->progress_user_data,
 					       (GAsyncReadyCallback) pk_task_ready_cb, state);
-	} else if (state->role == PK_ROLE_ENUM_ROLLBACK) {
-		pk_client_rollback_async (PK_CLIENT(state->task), state->transaction_id,
-					  state->cancellable, state->progress_callback, state->progress_user_data,
-					  (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_GET_REPO_LIST) {
 		pk_client_get_repo_list_async (PK_CLIENT(state->task), state->filters,
 					       state->cancellable, state->progress_callback, state->progress_user_data,
@@ -275,9 +290,9 @@ pk_task_do_async_action (PkTaskState *state)
 					     state->cancellable, state->progress_callback, state->progress_user_data,
 					     (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else if (state->role == PK_ROLE_ENUM_REPAIR_SYSTEM) {
-		pk_client_repair_system_async (PK_CLIENT(state->task), state->only_trusted,
-		                               state->cancellable, state->progress_callback, state->progress_user_data,
-		                               (GAsyncReadyCallback) pk_task_ready_cb, state);
+		pk_client_repair_system_async (PK_CLIENT(state->task), transaction_flags,
+					       state->cancellable, state->progress_callback, state->progress_user_data,
+					       (GAsyncReadyCallback) pk_task_ready_cb, state);
 	} else {
 		g_assert_not_reached ();
 	}
@@ -300,6 +315,8 @@ pk_task_package_filter_cb (PkPackage *package, gpointer user_data)
 	return TRUE;
 }
 
+static void pk_task_do_async_simulate_action (PkTaskState *state);
+
 /**
  * pk_task_simulate_ready_cb:
  **/
@@ -308,7 +325,6 @@ pk_task_simulate_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskStat
 {
 	PkTaskClass *klass = PK_TASK_GET_CLASS (state->task);
 	GError *error = NULL;
-	GPtrArray *array_messages = NULL;
 	guint i;
 	guint length;
 	PkPackageSack *sack = NULL;
@@ -343,6 +359,15 @@ pk_task_simulate_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskStat
 
 	/* get exit code */
 	state->exit_enum = pk_results_get_exit_code (state->results);
+	if (state->exit_enum == PK_EXIT_ENUM_NEED_UNTRUSTED) {
+		g_debug ("retrying with !only-trusted");
+		pk_bitfield_remove (state->transaction_flags,
+				    PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
+		/* retry this */
+		pk_task_do_async_simulate_action (state);
+		goto out;
+	}
+
 	if (state->exit_enum != PK_EXIT_ENUM_SUCCESS) {
 		/* we 'fail' with success so the appication gets a
 		 * chance to process the PackageKit-specific
@@ -360,7 +385,8 @@ pk_task_simulate_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskStat
 	untrusted_sack = pk_package_sack_filter_by_info (sack, PK_INFO_ENUM_UNTRUSTED);
 	if (pk_package_sack_get_size (untrusted_sack) > 0) {
 		g_debug ("we got an untrusted message, so skipping only-trusted");
-		state->only_trusted = FALSE;
+		pk_bitfield_remove (state->transaction_flags,
+				    PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	}
 
 	/* remove all the packages we want to ignore */
@@ -369,7 +395,7 @@ pk_task_simulate_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskStat
 	/* remove all the original packages from the sack */
 	if (state->package_ids != NULL) {
 		length = g_strv_length (state->package_ids);
-		for (i=0; i<length; i++)
+		for (i = 0; i < length; i++)
 			pk_package_sack_remove_package_by_id (sack, state->package_ids[i]);
 	}
 
@@ -385,8 +411,6 @@ pk_task_simulate_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskStat
 	/* run the callback */
 	klass->simulate_question (state->task, state->request, state->results);
 out:
-	if (array_messages != NULL)
-		g_ptr_array_unref (array_messages);
 	if (results != NULL)
 		g_object_unref (results);
 	if (sack != NULL)
@@ -402,40 +426,69 @@ out:
 static void
 pk_task_do_async_simulate_action (PkTaskState *state)
 {
+	PkBitfield transaction_flags = state->transaction_flags;
+
 	/* so the callback knows if we are serious or not */
+	pk_bitfield_add (transaction_flags, PK_TRANSACTION_FLAG_ENUM_SIMULATE);
 	state->simulate = TRUE;
 
 	/* do the correct action */
 	if (state->role == PK_ROLE_ENUM_INSTALL_PACKAGES) {
 		/* simulate install async */
 		g_debug ("doing install");
-		pk_client_simulate_install_packages_async (PK_CLIENT(state->task), state->package_ids,
-							   state->cancellable, state->progress_callback, state->progress_user_data,
-							   (GAsyncReadyCallback) pk_task_simulate_ready_cb, state);
+		pk_client_install_packages_async (PK_CLIENT(state->task),
+						  transaction_flags,
+						  state->package_ids,
+						  state->cancellable,
+						  state->progress_callback,
+						  state->progress_user_data,
+						  (GAsyncReadyCallback) pk_task_simulate_ready_cb,
+						  state);
 	} else if (state->role == PK_ROLE_ENUM_UPDATE_PACKAGES) {
 		/* simulate update async */
 		g_debug ("doing update");
-		pk_client_simulate_update_packages_async (PK_CLIENT(state->task), state->package_ids,
-							  state->cancellable, state->progress_callback, state->progress_user_data,
-							  (GAsyncReadyCallback) pk_task_simulate_ready_cb, state);
+		pk_client_update_packages_async (PK_CLIENT(state->task),
+						 transaction_flags,
+						 state->package_ids,
+						 state->cancellable,
+						 state->progress_callback,
+						 state->progress_user_data,
+						 (GAsyncReadyCallback) pk_task_simulate_ready_cb,
+						 state);
 	} else if (state->role == PK_ROLE_ENUM_REMOVE_PACKAGES) {
 		/* simulate remove async */
 		g_debug ("doing remove");
-		pk_client_simulate_remove_packages_async (PK_CLIENT(state->task), state->package_ids, state->autoremove,
-							  state->cancellable, state->progress_callback, state->progress_user_data,
-							  (GAsyncReadyCallback) pk_task_simulate_ready_cb, state);
+		pk_client_remove_packages_async (PK_CLIENT(state->task),
+						 transaction_flags,
+						 state->package_ids,
+						 state->allow_deps,
+						 state->autoremove,
+						 state->cancellable,
+						 state->progress_callback,
+						 state->progress_user_data,
+						 (GAsyncReadyCallback) pk_task_simulate_ready_cb,
+						 state);
 	} else if (state->role == PK_ROLE_ENUM_INSTALL_FILES) {
 		/* simulate install async */
 		g_debug ("doing install files");
-		pk_client_simulate_install_files_async (PK_CLIENT(state->task), state->files,
-						        state->cancellable, state->progress_callback, state->progress_user_data,
-						        (GAsyncReadyCallback) pk_task_simulate_ready_cb, state);
+		pk_client_install_files_async (PK_CLIENT(state->task),
+					       transaction_flags,
+					       state->files,
+					       state->cancellable,
+					       state->progress_callback,
+					       state->progress_user_data,
+					       (GAsyncReadyCallback) pk_task_simulate_ready_cb,
+					       state);
 	} else if (state->role == PK_ROLE_ENUM_REPAIR_SYSTEM) {
 		/* simulate repair system async */
 		g_debug ("doing repair system");
-		pk_client_simulate_repair_system_async (PK_CLIENT(state->task),
-		                                        state->cancellable, state->progress_callback, state->progress_user_data,
-		                                        (GAsyncReadyCallback) pk_task_simulate_ready_cb, state);
+		pk_client_repair_system_async (PK_CLIENT(state->task),
+					       transaction_flags,
+					       state->cancellable,
+					       state->progress_callback,
+					       state->progress_user_data,
+					       (GAsyncReadyCallback) pk_task_simulate_ready_cb,
+					       state);
 	} else {
 		g_assert_not_reached ();
 	}
@@ -738,6 +791,18 @@ pk_task_user_declined (PkTask *task, guint request)
 	return TRUE;
 }
 
+/**
+ * pk_task_retry_cancelled_transaction_cb:
+ **/
+static gboolean
+pk_task_retry_cancelled_transaction_cb (gpointer user_data)
+{
+	PkTaskState *state = (PkTaskState *) user_data;
+	pk_task_do_async_action (state);
+	state->retry_id = 0;
+	return FALSE;
+}
+
 /**
  * pk_task_ready_cb:
  **/
@@ -771,7 +836,8 @@ pk_task_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskState *state)
 
 	/* need untrusted */
 	if (state->exit_enum == PK_EXIT_ENUM_NEED_UNTRUSTED) {
-		state->only_trusted = FALSE;
+		pk_bitfield_remove (state->transaction_flags,
+				    PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 
 		/* running non-interactive */
 		if (!state->task->priv->interactive) {
@@ -866,9 +932,11 @@ pk_task_ready_cb (GObject *source_object, GAsyncResult *res, PkTaskState *state)
 		goto out;
 	}
 
-	/* just re-run the transaction */
+	/* just re-run the transaction after a small delay */
 	if (state->exit_enum == PK_EXIT_ENUM_CANCELLED_PRIORITY) {
-		pk_task_do_async_action (state);
+		state->retry_id = g_timeout_add (PK_TASK_TRANSACTION_CANCELLED_RETRY_TIMEOUT,
+						 pk_task_retry_cancelled_transaction_cb,
+						 state);
 		goto out;
 	}
 
@@ -922,7 +990,7 @@ pk_task_install_packages_async (PkTask *task, gchar **package_ids, GCancellable
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->package_ids = g_strdupv (package_ids);
 	state->request = pk_task_generate_request_id ();
 
@@ -974,7 +1042,7 @@ pk_task_update_packages_async (PkTask *task, gchar **package_ids, GCancellable *
 	state->task = g_object_ref (task);
 	if (cancellable != NULL)
 		state->cancellable = g_object_ref (cancellable);
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->package_ids = g_strdupv (package_ids);
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
@@ -1088,7 +1156,7 @@ pk_task_install_files_async (PkTask *task, gchar **files, GCancellable *cancella
 	state->task = g_object_ref (task);
 	if (cancellable != NULL)
 		state->cancellable = g_object_ref (cancellable);
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->files = g_strdupv (files);
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
@@ -1106,59 +1174,6 @@ pk_task_install_files_async (PkTask *task, gchar **files, GCancellable *cancella
 	g_object_unref (res);
 }
 
-/**
- * pk_task_update_system_async:
- * @task: a valid #PkTask instance
- * @cancellable: a #GCancellable or %NULL
- * @progress_callback: (scope call): the function to run when the progress changes
- * @progress_user_data: data to pass to @progress_callback
- * @callback_ready: the function to run on completion
- * @user_data: the data to pass to @callback_ready
- *
- * Update all the packages on the system with the highest versions found in all
- * repositories.
- * NOTE: you can't choose what repositories to update from, but you can do:
- * - pk_task_repo_disable()
- * - pk_task_update_system()
- * - pk_task_repo_enable()
- *
- * Since: 0.5.2
- **/
-void
-pk_task_update_system_async (PkTask *task, GCancellable *cancellable,
-			     PkProgressCallback progress_callback, gpointer progress_user_data,
-			     GAsyncReadyCallback callback_ready, gpointer user_data)
-{
-	GSimpleAsyncResult *res;
-	PkTaskState *state;
-
-	g_return_if_fail (PK_IS_CLIENT (task));
-	g_return_if_fail (callback_ready != NULL);
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	res = g_simple_async_result_new (G_OBJECT (task), callback_ready, user_data, pk_task_update_system_async);
-
-	/* save state */
-	state = g_slice_new0 (PkTaskState);
-	state->role = PK_ROLE_ENUM_UPDATE_SYSTEM;
-	state->res = g_object_ref (res);
-	state->task = g_object_ref (task);
-	if (cancellable != NULL)
-		state->cancellable = g_object_ref (cancellable);
-	state->only_trusted = TRUE;
-	state->progress_callback = progress_callback;
-	state->progress_user_data = progress_user_data;
-	state->request = pk_task_generate_request_id ();
-
-	g_debug ("adding state %p", state);
-	g_ptr_array_add (task->priv->array, state);
-
-	/* start trusted install async */
-	pk_task_do_async_action (state);
-
-	g_object_unref (res);
-}
-
 /**
  * pk_task_resolve_async:
  * @task: a valid #PkTask instance
@@ -1198,7 +1213,7 @@ pk_task_resolve_async (PkTask *task, PkBitfield filters, gchar **packages, GCanc
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->packages = g_strdupv (packages);
 	state->request = pk_task_generate_request_id ();
@@ -1251,7 +1266,7 @@ pk_task_search_names_async (PkTask *task, PkBitfield filters, gchar **values, GC
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->values = g_strdupv (values);
 	state->request = pk_task_generate_request_id ();
@@ -1304,7 +1319,7 @@ pk_task_search_details_async (PkTask *task, PkBitfield filters, gchar **values,
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->values = g_strdupv (values);
 	state->request = pk_task_generate_request_id ();
@@ -1357,7 +1372,7 @@ pk_task_search_groups_async (PkTask *task, PkBitfield filters, gchar **values, G
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->values = g_strdupv (values);
 	state->request = pk_task_generate_request_id ();
@@ -1410,7 +1425,7 @@ pk_task_search_files_async (PkTask *task, PkBitfield filters, gchar **values, GC
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->values = g_strdupv (values);
 	state->request = pk_task_generate_request_id ();
@@ -1462,7 +1477,7 @@ pk_task_get_details_async (PkTask *task, gchar **package_ids, GCancellable *canc
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->package_ids = g_strdupv (package_ids);
 	state->request = pk_task_generate_request_id ();
 
@@ -1513,7 +1528,7 @@ pk_task_get_update_detail_async (PkTask *task, gchar **package_ids, GCancellable
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->package_ids = g_strdupv (package_ids);
 	state->request = pk_task_generate_request_id ();
 
@@ -1565,7 +1580,7 @@ pk_task_download_packages_async (PkTask *task, gchar **package_ids, const gchar
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->package_ids = g_strdupv (package_ids);
 	state->directory = g_strdup (directory);
 	state->request = pk_task_generate_request_id ();
@@ -1617,7 +1632,7 @@ pk_task_get_updates_async (PkTask *task, PkBitfield filters, GCancellable *cance
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->request = pk_task_generate_request_id ();
 
@@ -1670,7 +1685,7 @@ pk_task_get_depends_async (PkTask *task, PkBitfield filters, gchar **package_ids
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->package_ids = g_strdupv (package_ids);
 	state->recursive = recursive;
@@ -1723,7 +1738,7 @@ pk_task_get_packages_async (PkTask *task, PkBitfield filters, GCancellable *canc
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->request = pk_task_generate_request_id ();
 
@@ -1776,7 +1791,7 @@ pk_task_get_requires_async (PkTask *task, PkBitfield filters, gchar **package_id
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->package_ids = g_strdupv (package_ids);
 	state->recursive = recursive;
@@ -1831,7 +1846,7 @@ pk_task_what_provides_async (PkTask *task, PkBitfield filters, PkProvidesEnum pr
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->provides = provides;
 	state->values = g_strdupv (values);
@@ -1884,7 +1899,7 @@ pk_task_get_files_async (PkTask *task, gchar **package_ids, GCancellable *cancel
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->package_ids = g_strdupv (package_ids);
 	state->request = pk_task_generate_request_id ();
 
@@ -1934,7 +1949,7 @@ pk_task_get_categories_async (PkTask *task, GCancellable *cancellable,
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->request = pk_task_generate_request_id ();
 
 	g_debug ("adding state %p", state);
@@ -1984,7 +1999,7 @@ pk_task_refresh_cache_async (PkTask *task, gboolean force, GCancellable *cancell
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->force = force;
 	state->request = pk_task_generate_request_id ();
 
@@ -1997,57 +2012,6 @@ pk_task_refresh_cache_async (PkTask *task, gboolean force, GCancellable *cancell
 	g_object_unref (res);
 }
 
-/**
- * pk_task_rollback_async:
- * @task: a valid #PkTask instance
- * @transaction_id: The transaction ID of the old transaction
- * @cancellable: a #GCancellable or %NULL
- * @progress_callback: (scope call): the function to run when the progress changes
- * @progress_user_data: data to pass to @progress_callback
- * @callback_ready: the function to run on completion
- * @user_data: the data to pass to @callback
- *
- * Rollback to a previous package state.
- *
- * Since: 0.6.5
- **/
-void
-pk_task_rollback_async (PkTask *task, const gchar *transaction_id, GCancellable *cancellable,
-			PkProgressCallback progress_callback, gpointer progress_user_data,
-			GAsyncReadyCallback callback_ready, gpointer user_data)
-{
-	GSimpleAsyncResult *res;
-	PkTaskState *state;
-
-	g_return_if_fail (PK_IS_TASK (task));
-	g_return_if_fail (callback_ready != NULL);
-	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
-
-	res = g_simple_async_result_new (G_OBJECT (task), callback_ready, user_data, pk_task_install_packages_async);
-
-	/* save state */
-	state = g_slice_new0 (PkTaskState);
-	state->role = PK_ROLE_ENUM_ROLLBACK;
-	state->res = g_object_ref (res);
-	state->task = g_object_ref (task);
-	if (cancellable != NULL)
-		state->cancellable = g_object_ref (cancellable);
-	state->progress_callback = progress_callback;
-	state->progress_user_data = progress_user_data;
-	state->ret = FALSE;
-	state->only_trusted = TRUE;
-	state->transaction_id = g_strdup (transaction_id);
-	state->request = pk_task_generate_request_id ();
-
-	g_debug ("adding state %p", state);
-	g_ptr_array_add (task->priv->array, state);
-
-	/* run task with callbacks */
-	pk_task_do_async_action (state);
-
-	g_object_unref (res);
-}
-
 /**
  * pk_task_get_repo_list_async:
  * @task: a valid #PkTask instance
@@ -2086,7 +2050,7 @@ pk_task_get_repo_list_async (PkTask *task, PkBitfield filters, GCancellable *can
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->filters = filters;
 	state->request = pk_task_generate_request_id ();
 
@@ -2138,7 +2102,7 @@ pk_task_repo_enable_async (PkTask *task, const gchar *repo_id, gboolean enabled,
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->ret = FALSE;
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->repo_id = g_strdup (repo_id);
 	state->enabled = enabled;
 	state->request = pk_task_generate_request_id ();
@@ -2166,9 +2130,12 @@ pk_task_repo_enable_async (PkTask *task, const gchar *repo_id, gboolean enabled,
  * Since: 0.7.2
  **/
 void
-pk_task_repair_system_async (PkTask *task, GCancellable *cancellable,
-                             PkProgressCallback progress_callback, gpointer progress_user_data,
-                             GAsyncReadyCallback callback_ready, gpointer user_data)
+pk_task_repair_system_async (PkTask *task,
+			     GCancellable *cancellable,
+			     PkProgressCallback progress_callback,
+			     gpointer progress_user_data,
+			     GAsyncReadyCallback callback_ready,
+			     gpointer user_data)
 {
 	GSimpleAsyncResult *res;
 	PkTaskState *state;
@@ -2187,7 +2154,7 @@ pk_task_repair_system_async (PkTask *task, GCancellable *cancellable,
 	state->task = g_object_ref (task);
 	if (cancellable != NULL)
 		state->cancellable = g_object_ref (cancellable);
-	state->only_trusted = TRUE;
+	state->transaction_flags = pk_bitfield_value (PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED);
 	state->progress_callback = progress_callback;
 	state->progress_user_data = progress_user_data;
 	state->request = pk_task_generate_request_id ();
@@ -2267,6 +2234,40 @@ pk_task_get_simulate (PkTask *task)
 	return task->priv->simulate;
 }
 
+/**
+ * pk_task_set_only_download:
+ * @task: a valid #PkTask instance
+ * @only_download: %FALSE to actually commit the transaction
+ *
+ * If the transaction should be prepared (depsolved, packages
+ * downloaded, etc) but not committed.
+ *
+ * Since: 0.8.1
+ **/
+void
+pk_task_set_only_download (PkTask *task, gboolean only_download)
+{
+	g_return_if_fail (PK_IS_TASK (task));
+	task->priv->only_download = only_download;
+	g_object_notify (G_OBJECT (task), "only-download");
+}
+
+/**
+ * pk_task_get_only_download:
+ * @task: a valid #PkTask instance
+ *
+ * Gets if we are just preparing the transaction for later.
+ *
+ * Return value: %TRUE if we are simulating
+ *
+ * Since: 0.8.1
+ **/
+gboolean
+pk_task_get_only_download (PkTask *task)
+{
+	g_return_val_if_fail (PK_IS_TASK (task), FALSE);
+	return task->priv->only_download;
+}
 
 /**
  * pk_task_set_interactive:
@@ -2319,6 +2320,9 @@ pk_task_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 	case PROP_INTERACTIVE:
 		g_value_set_boolean (value, priv->interactive);
 		break;
+	case PROP_ONLY_PREPARE:
+		g_value_set_boolean (value, priv->only_download);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -2341,6 +2345,9 @@ pk_task_set_property (GObject *object, guint prop_id, const GValue *value, GPara
 	case PROP_INTERACTIVE:
 		priv->interactive = g_value_get_boolean (value);
 		break;
+	case PROP_ONLY_PREPARE:
+		priv->only_download = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -2379,6 +2386,16 @@ pk_task_class_init (PkTaskClass *klass)
 				      G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_INTERACTIVE, pspec);
 
+	/**
+	 * PkTask:only-download:
+	 *
+	 * Since: 0.8.1
+	 */
+	pspec = g_param_spec_boolean ("only-download", NULL, NULL,
+				      FALSE,
+				      G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_ONLY_PREPARE, pspec);
+
 	g_type_class_add_private (klass, sizeof (PkTaskPrivate));
 }
 
