@@ -55,6 +55,8 @@ typedef struct
 
 GType		 pk_transaction_db_get_type		(void);
 PkTransactionDb	*pk_transaction_db_new			(void);
+gboolean	 pk_transaction_db_load			(PkTransactionDb	*tdb,
+							 GError			**error);
 gboolean	 pk_transaction_db_empty		(PkTransactionDb	*tdb);
 gboolean	 pk_transaction_db_add			(PkTransactionDb	*tdb,
 							 const gchar		*tid);
@@ -101,14 +103,6 @@ gboolean	 pk_transaction_db_set_proxy		(PkTransactionDb	*tdb,
 							 const gchar		*proxy_socks,
 							 const gchar		*no_proxy,
 							 const gchar		*pac);
-gboolean	 pk_transaction_db_get_root		(PkTransactionDb	*tdb,
-							 guint			 uid,
-							 const gchar		*session,
-							 gchar			**root);
-gboolean	 pk_transaction_db_set_root		(PkTransactionDb	*tdb,
-							 guint			 uid,
-							 const gchar		*session,
-							 const gchar		*root);
 
 G_END_DECLS
 
